@@ -113,7 +113,7 @@ Varinfo Interpreter::get_varinfo(Varcode code)
         TRACE("get_varinfo:applying %d increase of scale, ref, width\n", c_scale_ref_width_increase);
         scale += c_scale_ref_width_increase;
         bit_len += (10 * c_scale_ref_width_increase + 2) / 3;
-        bit_ref *= pow10[code];
+        bit_ref *= pow10[c_scale_ref_width_increase];
     }
 
     auto refval = c03_refval_overrides.find(code);
@@ -632,7 +632,7 @@ void Printer::b_variable(Varcode code)
 
 void Printer::c_modifier(Varcode code, Opcodes& next)
 {
-    print_lead(code);
+    print_lead(code); putc(' ', out);
     Interpreter::print_c_modifier(out, code, next);
     Interpreter::c_modifier(code, next);
 }
@@ -676,6 +676,13 @@ void Printer::define_bitmap(unsigned bitmap_size)
 {
 }
 
+unsigned Printer::define_associated_field_significance(Varinfo info)
+{
+    print_lead(info->code);
+    fprintf(out, " %s[%s]\n", info->desc, info->unit);
+    return 63;
+}
+
 unsigned Printer::define_bitmap_delayed_replication_factor(Varinfo info)
 {
     return 0;
