@@ -126,7 +126,7 @@ public:
     operator const T*() const { return reinterpret_cast<const T*>(addr); }
 
     template<typename T>
-    operator T*() const { return reinterpret_cast<T*>(addr); };
+    operator T*() const { return reinterpret_cast<T*>(addr); }
 };
 
 /**
@@ -196,6 +196,15 @@ public:
 
     size_t read(void* buf, size_t count);
 
+    /**
+     * Read `count` bytes into bufr, retrying partial reads, stopping at EOF.
+     *
+     * Return true if `count` bytes have been read, false in case of eof, and
+     * raise an exception in case EOF was found after reading between 0 and
+     * count-1 bytes.
+     */
+    bool read_all_or_retry(void* buf, size_t count);
+
     /**
      * Read all the data into buf, throwing runtime_error in case of a partial
      * read
@@ -272,13 +281,11 @@ public:
      */
     bool ofd_getlk(struct ::flock&);
 
-    /**
-     * Call sendfile with this file as in_fd, falling back on write if it is
-     * not available.
-     *
-     * Perform retry if data was partially written.
-     */
-    void sendfile(FileDescriptor& out_fd, off_t offset, size_t count);
+    /// Get open flags for the file
+    int getfl();
+
+    /// Set open flags for the file
+    void setfl(int flags);
 
     operator int() const { return fd; }
 };
@@ -415,20 +422,23 @@ struct Path : public ManagedNamedFileDescriptor
     /**
      * Open the given pathname with flags | O_PATH.
      */
-    Path(const char* pathname, int flags=0);
+    Path(const char* pathname, int flags=0, mode_t mode=0777);
     /**
      * Open the given pathname with flags | O_PATH.
      */
-    Path(const std::string& pathname, int flags=0);
+    Path(const std::string& pathname, int flags=0, mode_t mode=0777);
     /**
      * Open the given pathname calling parent.openat, with flags | O_PATH
      */
-    Path(Path& parent, const char* pathname, int flags=0);
+    Path(Path& parent, const char* pathname, int flags=0, mode_t mode=0777);
     Path(const Path&) = delete;
     Path(Path&&) = default;
     Path& operator=(const Path&) = delete;
     Path& operator=(Path&&) = default;
 
+    /// Wrapper around open(2) with flags | O_PATH
+    void open(int flags, mode_t mode=0777);
+
     DIR* fdopendir();
 
     /// Begin iterator on all directory entries
@@ -457,15 +467,25 @@ struct Path : public ManagedNamedFileDescriptor
 
     void unlinkat(const char* pathname);
 
+    void mkdirat(const char* pathname, mode_t mode=0777);
+
     /// unlinkat with the AT_REMOVEDIR flag set
     void rmdirat(const char* pathname);
 
+    void symlinkat(const char* target, const char* linkpath);
+
+    std::string readlinkat(const char* pathname);
+
     /**
      * Delete the directory pointed to by this Path, with all its contents.
      *
      * The path must point to a directory.
      */
     void rmtree();
+
+    static std::string mkdtemp(const std::string& prefix);
+    static std::string mkdtemp(const char* prefix);
+    static std::string mkdtemp(char* pathname_template);
 };
 
 
@@ -530,6 +550,28 @@ public:
 };
 
 
+/**
+ * Open a temporary directory.
+ *
+ * By default, the temporary directory will be deleted when the object is
+ * deleted.
+ */
+class Tempdir : public Path
+{
+protected:
+    bool m_rmtree_on_exit = true;
+
+public:
+    Tempdir();
+    Tempdir(const std::string& prefix);
+    Tempdir(const char* prefix);
+    ~Tempdir();
+
+    /// Change the rmtree-on-exit behaviour
+    void rmtree_on_exit(bool val);
+};
+
+
 /// Read whole file into memory. Throws exceptions on failure.
 std::string read_file(const std::string &file);
 
